'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var rollup = require('rollup');
var core = require('onebuild/core');
var resolve = _interopDefault(require('@rollup/plugin-node-resolve'));
var alias = _interopDefault(require('@rollup/plugin-alias'));
var builtins = _interopDefault(require('builtin-modules'));
var models = require('onebuild/models');

const ExternalDefault = [...builtins, "builtin-modules", "chalk", "console-timestamp", "rollup", "@rollup/plugin-node-resolve", "@rollup/plugin-alias"];
class OneBuildRollupPlugin {
  constructor() {
    this.name = "RollupBundler";
    this.rollupOptions = Object.assign({});
  }
  configure(instance) {
    this.pluginManager = instance;
    this.configureOptions = this.configureOptions || Object.assign({});
    this.configureOptions.end = () => this.pluginManager;
    this.configureOptions.addOptions = (options) => {
      this.rollupOptions = options;
      return this.configureOptions;
    };
    return this.configureOptions;
  }
  async execute(ctx, next) {
    const startTime = new Date().getTime();
    core.Utils.logTime("Rollup running...", startTime);
    const rollupOptions = this.mergedOptions;
    const outputOptions = Array.isArray(this.rollupOptions.output) ? this.rollupOptions.output : [this.rollupOptions.output];
    const composers = ctx.result[models.Constants.Plugins.ComposerPlugin];
    if (composers && composers.length > 0) {
      rollupOptions.input = rollupOptions.input || {};
      let manualChunks = {};
      composers.forEach((composer) => {
        composer.forEach((manifest) => {
          rollupOptions.input[`${manifest.name || ""}_${manifest.id.toString()}`] = manifest.entry;
          if (manifest.includePaths) {
            manualChunks[`${manifest.name || ""}_${manifest.id.toString()}`] = [...manifest.includePaths, manifest.entry];
          }
        });
      });
      outputOptions.forEach((output) => output.manualChunks = Object.assign({}, rollupOptions.manualChunks || {}, manualChunks));
    }
    const bundle = await rollup.rollup(rollupOptions);
    await core.Utils.asyncForEach(outputOptions, async (outputOption) => {
      await bundle.write(outputOption);
    });
    core.Utils.logTime("Rollup done", startTime);
    await next();
  }
  get mergedOptions() {
    const options = Object.assign({preserveEntrySignatures: "allow-extension"}, this.rollupOptions || {});
    const plugins = options.plugins || [];
    if (this.rollupOptions.includeDefaultPlugins !== false) {
      plugins.unshift(resolve({
        extensions: [".mjs", ".js", ".jsx", ".ts", ".tsx", ".json", ".node"]
      }));
      if (this.rollupOptions.alias) {
        plugins.unshift(alias(this.rollupOptions.alias));
      }
      if (options.external) {
        if (Array.isArray(options.external)) {
          options.external = options.external.concat(ExternalDefault);
        }
      } else {
        options.external = ExternalDefault;
      }
    }
    options.plugins = plugins;
    delete options.alias;
    return options;
  }
}

exports.ExternalDefault = ExternalDefault;
exports.OneBuildRollupPlugin = OneBuildRollupPlugin;
